/// <reference types="node" />
import { IModLoaderAPI } from "modloader64_api/IModLoaderAPI";
export declare class ScriptKiddy {
    ModLoader: IModLoaderAPI;
    constructor(ModLoader: IModLoaderAPI);
    ErrorCheck_CodeSize(size: number): void;
    CodeBuffer_To_Data(buf: Buffer): string;
    /**
     *
     * @param addr address to place J
     * @param func address to J to
     */
    J(addr: number, func: number): void;
    /**
     *
     * @param addr address to place JAL
     * @param func address to JAL to
     */
    JAL(addr: number, func: number): void;
    /**
     *
     * @param count number of nops to generate
     * @return the generated nops to assemble
    */
    NOP(count: number): string;
    /**
     *
     * @param addr address to place J
     * @param func address to J to
     */
    JNOP(addr: number, func: number): void;
    /**
     *
     * @param addr address to place JAL
     * @param func address to JAL to
     */
    JALNOP(addr: number, func: number): void;
    /**
     * Create a codecave using J, where the original code executes before the arbitrary code
     * @param addr address to codecave
     * @param code assembly to assemble into the cave
     * @returns pointer to codecave code
     */
    CODECAVE_PRE_J(addr: number, code: string): number;
    /**
     * Create a codecave using J, where the original code executes after the arbitrary code
     * @param addr address to codecave
     * @param code assembly to assemble into the cave
     * @returns pointer to codecave code
     */
    CODECAVE_POST_J(addr: number, code: string): number;
    /**
     * Create a codecave using JAL, where the original code executes before the arbitrary code
     * @param addr address to codecave
     * @param code assembly to assemble into the cave
     * @returns pointer to codecave code
     */
    CODECAVE_PRE_JAL(addr: number, code: string): number;
    /**
     * Create a codecave using JAL, where the original code executes after the arbitrary code
     * @param addr address to codecave
     * @param code assembly to assemble into the cave
     * @returns pointer to codecave code
     */
    CODECAVE_POST_JAL(addr: number, code: string): number;
    /**
     * Create a codecave using J, where the original code does not execute
     * @param addr address to codecave
     * @param code assembly to assemble into the cave
     * @returns pointer to codecave code
     */
    CODECAVE_J(addr: number, code: string): number;
    /**
     * Create a codecave using JAL, where the original code does not execute
     * @param addr address to codecave
     * @param code assembly to assemble into the cave
     * @returns pointer to codecave code
     */
    CODECAVE_JAL(addr: number, code: string): number;
}
//# sourceMappingURL=scriptkiddy.d.ts.map